#pragma once
#include "precompile.h"
#include "error.h"
#include "message.h"
// en inline funktion aka copy pasta funktion placerar en kopia av funktion dr den kallas
inline void startup_wsa() { // anvnds fr att initializera windows sockets till verision 2.2
	WSAData ws;
	Error wsa = WSAStartup(MAKEWORD(2, 2), &ws);
	//std::cout << "WSAStartup() - " << wsa << std::endl;
	if (wsa) {
		exit(-1);
	}
}

using response_table = std::unordered_map <std::string, std::function<std::string(std::string)>>;
using action_table = std::unordered_map <std::string, std::function<void(std::string)>>;

// klass fr att hantera motagning och skickning av data till servern. ansvarar inte fr att upperthlla en ansutning, "superklassen" r ansvarig fr det
class tcp_client
{
public:
	/// <summary>
	/// Ny klient som ska anslutas till 'ip' och 'port'
	/// </summary>
	/// <param name="ip">anslutnings address</param>
	/// <param name="port">anslutnings port</param>
	tcp_client(std::string, int);

	/// <summary>
	/// Frsk ansluta med ip och port som angivits 
	/// Undantag om misslyckande
	/// </summary>
	/// <returns>om anslutningen lyckades</returns>
	bool connect();

	/// <summary>
	/// Loop fr att ta emot paket
	/// </summary>
	void paket_loop();

	// funktioner fr enkapsulering
	void set_ip(std::string ip) { ip_ = ip; }
	void set_port(int port) { port_ = port; }
	std::string get_ip() { return ip_; }
	int get_port() { return port_; }
	void set_disconnect_action(std::function<void()> dc) { disconnect_function = dc; }

	void set_responses(response_table rt) { response_map = rt; }
	void set_actions(action_table at) { action_map = at; }

	/// <summary>
	/// Skicka ett message till servern
	/// </summary>
	/// <param name="meddelande">meddelande att skicka</param>
	/// <returns>True: inga errors, False: send error</returns>
	bool send(message);

	/// <summary>
	/// Ta emot ett psuedo tcp paket 
	/// Private eftersom att man fr manipulera packet_queuen istllet. 
	/// </summary>
	packet receive_packet();

private:

	bool alive;

	// "hash" tabeller fr inkommande paket
	response_table response_map; // fr paket som endast krver en respons, tex server skickar: ge mig ditt anvndarnamn. klienten skickar anvndarnamn. utbytes trafik
	action_table action_map; // fr paket som krver en egen funktion. tex servern skickar olika cmd-commandon. vi skapar en cmd prompt och exekeverar kommandon i en loop. utbytes trafik fast med mer n ett paket

	//kommer att anvndas fr att ominitialicera instansen av tcp_client eftersom att state baserade funktioner inte kommer att funka nr servern kommer online igen
	std::function<void()> disconnect_function; // kallas nr anslutningen upphr

	SOCKET server_socket; // soket som r ansluten till server

	std::string ip_;
	int port_;

	bool is_ip(std::string); // fr att verifiera att en strng r en ip address
	std::string dns2string(std::string); // konvertera mellan dns och ip address. tex example.com -> 123.123.123.123

	/// <summary>
	/// Ta emot 'size' antal bytes frn servern
	/// </summary>
	/// <param name="size">hur mnga bytes som ska tas emot</param>
	/// <returns>strng med datan som togs emot</returns>
	std::string receive_bytes(int);

	/// <summary>
	/// Ta emot en size antal bytes frn servern och returnera det som en strng
	/// </summary>
	/// <param name="size">antal bytes som ska tas emot</param>
	/// <returns>strngen som togs emot</returns>
	std::string receive(int);

	/// <summary>
	/// Ta emot psuedo huvudet p packetet och verifera syntaxen p den
	/// </summary>
	/// <returns>header struktur fr ltt manipulering av huvudet</returns>
	header receive_header();

	/// <summary>
	/// Verifera om socketen r lsbar / om det finns data att ta emot frn socketen
	/// </summary>
	/// <param name="sock">socketen som ska testas</param>
	/// <param name="sec">sekunder som man ska vnta</param>
	/// <param name="milli">millisekunder som man ska vnta</param>
	/// <returns>om socketen r lsbar</returns>
	bool readable(SOCKET, int = 0, int = 0);

	/// <summary>
	/// Hanterar paket i antingen response_map eller action_map
	/// </summary>
	/// <param name="paket">paket som ska hanteras</param>
	void handle_packet(packet);
};

