#pragma once
#include "precompile.h"
#include "client_list.h"
#include "Error.h"
#include "window_log.h"

// en inline funktion aka copy pasta funktion placerar en kopia av funktion dr den kallas
inline void startup_wsa() { // anvnds fr att initializera windows sockets till verision 2.2
	WSAData ws;
	Error wsa = WSAStartup(MAKEWORD(2, 2), &ws);
	//std::cout << "WSAStartup() - " << wsa << std::endl;
	if (wsa) {
		exit(-1);
	}
}

// klass fr att hantera anslutningar till olika klienter
class tcp_server
{

public:

	/// <summary>
	/// Ny server bunden till porten 'port'
	/// </summary>
	/// <param name="port">port att lyssna p</param>
	/// <param name="output_stream">ofstream att logga till, kan vara fil, konsol eller pip</param>
	tcp_server(int port, window_log* stream) : listen_port(port), log(stream), alive{ true } { clients.set_name("klient"); }

	// fr att enkapsulera logger, detta p grund av att det r en osker pekare
	void set_log_stream(window_log*);
	window_log* get_log_stream();

	/// <summary>
	/// F tkomst till paket kn fr manipulering av data
	/// </summary>
	/// <returns>Paketkn</returns>
	std::deque<packet>& get_queue();

	/// <summary>
	/// F tkomst till klient listan
	/// </summary>
	/// <returns>Klientlistan</returns>
	client_list& get_clients();

	/// <summary>
	/// Binda servern
	/// Kastar undantag om funktionen inte lyckas
	/// </summary>
	void bind();

	/// <summary>
	/// Lyssna efter anslutningar
	/// Kastar undantag om funktionen inte lyckas
	/// </summary>
	void listen();

	/// <summary>
	/// Starta packet och klient hanteraren i en separat trd
	/// </summary>
	void run();

	/// <summary>
	/// Skicka ett message till en klient
	/// </summary>
	/// <param name="klient">mlklienten som ska ta emot meddelandet</param>
	/// <param name="meddelande">meddelande att skicka</param>
	/// <returns>True: inga errors, False: send error</returns>
	bool send(client, message);

protected:

	bool alive;

	client_list clients{}; // lista av klienter

	std::deque<packet> packet_queue{}; // denna r en k eftersom att vi inte bryr oss om random access + [FIFO]

	const int listen_port; // denna r const eftersom att det inte finns ngon anledning att ndra p en lyssningsport 

	SOCKET listen_socket{0}; // tcp socket

	window_log* log; // strm fr output av information

	/// <summary>
	/// Fr tiden som en strng
	/// </summary>
	/// <returns>tiden</returns>
	std::string str_time();

	/// <summary>
	/// Ta emot ett psuedo tcp paket 
	/// Private eftersom att man fr manipulera packet_queuen istllet. 
	/// </summary>
	/// <param name="klient">klienten som paketet ska tas emot frn</param>
	/// <param name="size">storleken p paketet</param>
	packet receive_paket(client&);

	/// <summary>
	/// Ta emot 'size' antal bytes frn en klient
	/// </summary>
	/// <param name="klient">klient som bytes ska tas emot frn</param>
	/// <param name="size">hur mnga bytes som ska tas emot</param>
	/// <returns>strng med datan som togs emot</returns>
	std::string receive_bytes(client&, int);

	/// <summary>
	/// Ta emot en size antal bytes frn klienten och returnera det som en strng
	/// </summary>
	/// <param name="klient">klienten som paketet ska tas emot frn</param>
	/// <param name="size">antal bytes som ska tas emot</param>
	/// <returns>strngen som togs emot</returns>
	std::string receive(client&, int);

	/// <summary>
	/// Ta emot psuedo huvudet p packetet och verifera syntaxen p den
	/// </summary>
	/// <param name="klient">klienten som huvudet ska tas ifrn</param>
	/// <returns>header struktur fr ltt manipulering av huvudet</returns>
	header receive_header(client&);

	/// <summary>
	/// Verifera om socketen r lsbar / om det finns data att ta emot frn socketen
	/// </summary>
	/// <param name="sock">socketen som ska testas</param>
	/// <param name="sec">sekunder som man ska vnta</param>
	/// <param name="milli">millisekunder som man ska vnta</param>
	/// <returns>om socketen r lsbar</returns>
	bool readable(SOCKET, int = 0, int = 0);

	/// <summary>
	/// Acceptera den nydligen anslutna klienten (om den authenticerar sig)(TODO)
	/// </summary>
	void accept_user();

	/// <summary>
	/// Ta bort ett packet ur kn
	/// </summary>
	/// <param name="paket">paket som ska tas bort</param>
	void delete_packet(packet);

};

